/********************************************************************************** 
 The SJF2440 is based on SJF2410, which was written by Inwook Kong(purnnamu).  
 The main difference between SJF2440 and SJF2410 is BSC(Boundary Scan Cell)Index.
 Refer to S3C2440_jtag_buf.bsd file.
 Thanks to Inwook Kong. 
 
 Revision history
 2003.09.16: ver 0.1 (Y.H.Lee)
  - SEC JTAG FLASH Program for S3C2440X & SMDK2440 B/D.
  - K9S1208, AMD 29LV800BB and E28F128 StrataFlash programming is supported.
  - External peripherals(PD6710,...) read/write is supported.
**********************************************************************************/ 

//SJF2440 is derived from following SJF2410(SJF)...
   
/********************************************************************************** 
 The SJF is written by analyzing ezTAG program, which was written by Jaewook Cheong,
 SEC S/W Centor. 
 Special thanks to Jaewook Cheong and Dale Roberts(the author of GIVEIO.sys)

 In SJF, the following feature is updated from ezTAG

 1. The structure of JTAG functions is changed for speed-up.
 2. The indexs of the pins become coherent with the BSDL file.
 3. SAMPLE/PRELOAD is used for initializing outCellValue[].
 4. The array size error is fixed(S3C2410_MAX_CELLS -> S3C2410_MAX_CELL_INDEX+2)
 5. The array was not followed by '\0', which is fixed.
 6. JTAG_ID reading error is fixed.
 7. Support K9S1208 SMD card for the SMDK2410 board.
 8. The programming speed is enhanced.
 **********************************************************************************/
/*******************************************
 Revision history
 2002.06.10:purnnamu:ver 0.1
  -first release
 2002.06.21:purnnamu:ver 0.11
  -trivial display error is fixed.
 2002.08.20:purnnamu:ver 0.3
  -Strata flash is supported.
 2002.08.20:purnnamu:ver 0.4
  -AM29LV800BB flash is supported.
 *******************************************/

#include <windows.h>
#include <stdio.h>
#include <conio.h>
#include "def.h"
#include "pin2410.h"
#include "jtag.h"
#include "ppt.h"
#include "k9s1208.h"
#include "strata32.h"
#include "am29f800.h"
#include "mem_rdwr.h"
#include "sjf2440.h"

FILE *stream;
U32 imageSize;

void OpenPpt(void);

void ErrorBanner(void);

static void *function[]=
{
    "K9F5608 prog    ",
	"K9F1208 prog    ",
    "K9F1G08 prog    ",
	"28F128J3A prog  ",
    "AM29LV800 Prog  ",
    "Memory Rd/Wr    ",
    "Exit            ",
    0
};

#include "stdio.h"
void gres(int v , char *prop , char *log )
{
	FILE *f = fopen("log.log","wt");
	fprintf(f,"%d\n%s\n%s",v,prop,log);
	fclose(f);
}



int main(int argc,char *argv[])
{
    char num=0;
    int i;

    printf("\n");
    printf("+------------------------------------+\n");
    printf("|       XSJF JTAG FLASH v 0.1        |\n");
    printf("|       (S3C2410X & S3C2440X)        |\n");
    printf("+------------------------------------+\n");
    printf("Usage: SJF /f:<filename> /d=<delay>\n");

    delayLoopCount=100;
    srcFileName[0]='\0';

    for(i=1;i<argc;i++)
    {
	switch(argv[i][1])
	{
	case 'f':
	    strcpy(srcFileName,&(argv[i][3]));
	    break;
	case 'd':
	    delayLoopCount=atoi(&argv[i][3]);
	    break;
	default:
	    printf("ERROR: unknown option /%c is detected.\n", argv[i][1]);
	    break;
	}
    }
    //Open Parallel Port for JTAG Dongle Control  
    OpenPpt();
    
	//Take Image File Handler 
    if(srcFileName[0]!='\0')
	OpenImageFile(srcFileName);
	
	//Read 2440 IDCODE Register Value
    JTAG_ReadId();

    //Initialize S2410 Boundary Scan Cell
	S2410_InitCell();

	printf("\n[SJF Main Menu]\n");
    i=0;
    while(1)
    {
	//display menu
	printf("%2d:%s",i,function[i]);
	i++;
	if((int)(function[i])==0)
	{
	    printf("\n");
	    break;
	}
	//if((i%5)==0)
	    printf("\n");
    }
	 
    printf("Select the function to test:");
    scanf("%d",&i);
    switch(i)
    {
    case 0:
		//Samsung NAND Flash Programming 
		K9F5608_Menu();
	break;
	
	case 1:
		//Samsung NAND Flash Programming 
		K9S1208_Menu();
	break;

	case 2:
       	//Samsung NAND Flash Programming 
		K9F1G08_Menu();
	break;

    case 3:
       	//Intel STRATA NOR Flash(32bit) Programming 
		Program28F128J3A();
	break;

	case 4:
		//AMD NOR Flash(16bit) Programming 
       	ProgramAM29F800();
	break;

    case 5:
		//Read/Write Specific External Memory Address 
		MRW_Menu();
	break;

    default:
	return 0;
	break; //Exit menu
    }
    return 0;
}

//////////////////////////////////////////////////////////
void OpenImageFile(char *filename)
{
    U32 fileEnd,fileStart;
    stream = fopen(filename,"rb");
    if(stream==NULL)
    {
	printf("\nERROR:can't find the file.\n");
	exit(1);
    }

    fseek(stream,0L,SEEK_END);
    fileEnd=ftell(stream);
    fseek(stream,0L,SEEK_SET);
    fileStart=ftell(stream);

    imageSize=fileEnd-fileStart;  /* fileend == peof+1 */
}

int LoadImageFile(U8 *buf,int size)
{
    int i,readSize=size;
    for(i=0;i<size;i++)
    {
	if(feof(stream))
	{
	    readSize=i;
	    for(;i<size;i++)buf[i]=0;
	    break;
	}
	buf[i] = fgetc(stream);
    }
    return readSize;
}

void OpenPpt(void)
{
    if(!InstallGiveIo())
    {
        printf("ERROR: Couldn't open giveio.sys\n");
        exit(1);
    }

    validPpt = GetValidPpt();
    if(!validPpt)
    {
	printf("ERROR: Unable to find a parallel port\n");
	exit(1);
    }
    SetPptCompMode();	
}


